@@ -18,29 +18,92 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from __future__ import print_function
-import sip
-
-sip.setapi("QString", 2)
-sip.setapi("QVariant", 2)
-
-from PyQt4 import QtGui, QtCore
-
 import argparse
+from functools import partial
+from itertools import chain
+import logging
+from operator import attrgetter
 import os.path
 import platform
 import re
 import shutil
 import signal
 import sys
-from functools import partial
-from itertools import chain
+
+from PyQt5 import (
+    QtCore,
+    QtGui,
+    QtWidgets,
+)
+
+from picard import (
+    PICARD_APP_NAME,
+    PICARD_FANCY_VERSION_STR,
+    PICARD_ORG_NAME,
+    acoustid,
+    config,
+    log,
+)
+from picard.acoustid.manager import AcoustIDManager
+from picard.album import (
+    Album,
+    NatAlbum,
+)
+from picard.browser.browser import BrowserIntegration
+from picard.browser.filelookup import FileLookup
+from picard.cluster import (
+    Cluster,
+    ClusterList,
+    UnclusteredFiles,
+)
+from picard.collection import load_user_collections
+from picard.config_upgrade import upgrade_config
+from picard.const import (
+    USER_DIR,
+    USER_PLUGIN_DIR,
+)
+from picard.dataobj import DataObject
+from picard.disc import Disc
+from picard.file import File
+from picard.formats import open_ as open_file
+from picard.i18n import setup_gettext
+from picard.plugin import PluginManager
+from picard.releasegroup import ReleaseGroup
+from picard.track import (
+    NonAlbumTrack,
+    Track,
+)
+from picard.util import (
+    check_io_encoding,
+    decode_filename,
+    encode_filename,
+    is_hidden,
+    mbid_validate,
+    thread,
+    uniqify,
+    versions,
+    webbrowser2,
+)
+from picard.util.checkupdate import UpdateCheckManager
+from picard.webservice import WebService
+from picard.webservice.api_helpers import (
+    AcoustIdAPIHelper,
+    MBAPIHelper,
+)
+
+import picard.resources  # pylint: disable=unused-import
+
+from picard.ui.itemviews import BaseTreeView
+from picard.ui.mainwindow import MainWindow
+from picard.ui.searchdialog.album import AlbumSearchDialog
+from picard.ui.searchdialog.artist import ArtistSearchDialog
+from picard.ui.searchdialog.track import TrackSearchDialog
 
 
-# A "fix" for http://python.org/sf/1438480
-def _patched_shutil_copystat(src, dst):
+# A "fix" for https://bugs.python.org/issue1438480
+def _patched_shutil_copystat(src, dst, *, follow_symlinks=True):
     try:
-        _orig_shutil_copystat(src, dst)
+        _orig_shutil_copystat(src, dst, follow_symlinks=follow_symlinks)
     except OSError:
         pass
 
@@ -48,47 +111,8 @@ def _patched_shutil_copystat(src, dst):
 _orig_shutil_copystat = shutil.copystat
 shutil.copystat = _patched_shutil_copystat
 
-import picard.resources
-from picard.i18n import setup_gettext
 
-from picard import (PICARD_APP_NAME, PICARD_ORG_NAME, PICARD_FANCY_VERSION_STR,
-                    log, acoustid, config)
-from picard.album import Album, NatAlbum
-from picard.browser.browser import BrowserIntegration
-from picard.browser.filelookup import FileLookup
-from picard.cluster import Cluster, ClusterList, UnmatchedFiles
-from picard.const import USER_DIR, USER_PLUGIN_DIR
-from picard.dataobj import DataObject
-from picard.disc import Disc
-from picard.file import File
-from picard.formats import open as open_file
-from picard.track import Track, NonAlbumTrack
-from picard.releasegroup import ReleaseGroup
-from picard.collection import load_user_collections
-from picard.ui.mainwindow import MainWindow
-from picard.ui.itemviews import BaseTreeView
-from picard.plugin import PluginManager
-from picard.acoustidmanager import AcoustIDManager
-from picard.config_upgrade import upgrade_config
-from picard.util import (
-    decode_filename,
-    encode_filename,
-    thread,
-    mbid_validate,
-    check_io_encoding,
-    uniqify,
-    is_hidden,
-    versions,
-)
-from picard.webservice import XmlWebService
-from picard.ui.searchdialog import (
-    TrackSearchDialog,
-    AlbumSearchDialog,
-    ArtistSearchDialog
-)
-
-
-class Tagger(QtGui.QApplication):
+class Tagger(QtWidgets.QApplication):
 
     tagger_stats_changed = QtCore.pyqtSignal()
     listen_port_changed = QtCore.pyqtSignal(int)
@@ -99,22 +123,50 @@ class Tagger(QtGui.QApplication):
 
     __instance = None
 
+    _debug = False
+    _no_restore = False
+
     def __init__(self, picard_args, unparsed_args, localedir, autoupdate):
+
+        # Use the new fusion style from PyQt5 for a modern and consistent look
+        # across all OSes.
+        if sys.platform != "darwin":
+            self.setStyle('Fusion')
+
         # Set the WM_CLASS to 'MusicBrainz-Picard' so desktop environments
         # can use it to look up the app
-        QtGui.QApplication.__init__(self, ['MusicBrainz-Picard'] + unparsed_args)
+        super().__init__(['MusicBrainz-Picard'] + unparsed_args)
+        self.setDesktopFileName('org.musicbrainz.Picard.desktop')
         self.__class__.__instance = self
-
         config._setup(self, picard_args.config_file)
 
+        # Allow High DPI Support
+        self.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
+        self.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
+        super().setStyleSheet(
+            'QGroupBox::title { /* PICARD-1206, Qt bug workaround */ }'
+        )
+
         self._cmdline_files = picard_args.FILE
-        self._autoupdate = autoupdate
-        self._debug = False
+        self.autoupdate_enabled = autoupdate
+        self._no_restore = picard_args.no_restore
+        self._no_plugins = picard_args.no_plugins
+
+        self.set_log_level(config.setting['log_verbosity'])
+
+        if picard_args.debug or "PICARD_DEBUG" in os.environ:
+            self.set_log_level(logging.DEBUG)
 
         # FIXME: Figure out what's wrong with QThreadPool.globalInstance().
         # It's a valid reference, but its start() method doesn't work.
         self.thread_pool = QtCore.QThreadPool(self)
 
+        # Provide a separate thread pool for operations that should not be
+        # delayed by longer background processing tasks, e.g. because the user
+        # expects instant feedback instead of waiting for a long list of
+        # operations to finish.
+        self.priority_thread_pool = QtCore.QThreadPool(self)
+
         # Use a separate thread pool for file saving, with a thread count of 1,
         # to avoid race conditions in File._save_and_rename.
         self.save_thread_pool = QtCore.QThreadPool(self)
@@ -143,26 +195,17 @@ class Tagger(QtGui.QApplication):
             signal.signal(signal.SIGINT, self.signal)
             signal.signal(signal.SIGTERM, self.signal)
 
+        if sys.platform == "darwin":
+            # On macOS it is not common that the global menu shows icons
+            self.setAttribute(QtCore.Qt.AA_DontShowIconsInMenus)
+
         # Setup logging
-        self.debug(picard_args.debug or "PICARD_DEBUG" in os.environ)
         log.debug("Starting Picard from %r", os.path.abspath(__file__))
         log.debug("Platform: %s %s %s", platform.platform(),
                   platform.python_implementation(), platform.python_version())
         log.debug("Versions: %s", versions.as_string())
         log.debug("Configuration file path: %r", config.config.fileName())
 
-        # TODO remove this before the final release
-        if sys.platform == "win32":
-            olduserdir = "~\\Local Settings\\Application Data\\MusicBrainz Picard"
-        else:
-            olduserdir = "~/.picard"
-        olduserdir = os.path.expanduser(olduserdir)
-        if os.path.isdir(olduserdir):
-            log.info("Moving %r to %r", olduserdir, USER_DIR)
-            try:
-                shutil.move(olduserdir, USER_DIR)
-            except:
-                pass
         log.debug("User directory: %r", os.path.abspath(USER_DIR))
 
         # for compatibility with pre-1.3 plugins
@@ -178,7 +221,9 @@ class Tagger(QtGui.QApplication):
 
         upgrade_config()
 
-        self.xmlws = XmlWebService()
+        self.webservice = WebService()
+        self.mb_api = MBAPIHelper(self.webservice)
+        self.acoustid_api = AcoustIdAPIHelper(self.webservice)
 
         load_user_collections()
 
@@ -188,17 +233,16 @@ class Tagger(QtGui.QApplication):
 
         # Load plugins
         self.pluginmanager = PluginManager()
-        if hasattr(sys, "frozen"):
-            self.pluginmanager.load_plugindir(os.path.join(os.path.dirname(sys.argv[0]), "plugins"))
-        else:
-            mydir = os.path.dirname(os.path.abspath(__file__))
-            self.pluginmanager.load_plugindir(os.path.join(mydir, "plugins"))
-            self.pluginmanager.load_plugindir(os.path.join(mydir, os.pardir, "contrib", "plugins"))
+        if not self._no_plugins:
+            if hasattr(sys, "frozen"):
+                self.pluginmanager.load_plugindir(os.path.join(os.path.dirname(sys.argv[0]), "plugins"))
+            else:
+                mydir = os.path.dirname(os.path.abspath(__file__))
+                self.pluginmanager.load_plugindir(os.path.join(mydir, "plugins"))
 
-        if not os.path.exists(USER_PLUGIN_DIR):
-            os.makedirs(USER_PLUGIN_DIR)
-        self.pluginmanager.load_plugindir(USER_PLUGIN_DIR)
-        self.pluginmanager.query_available_plugins()
+            if not os.path.exists(USER_PLUGIN_DIR):
+                os.makedirs(USER_PLUGIN_DIR)
+            self.pluginmanager.load_plugindir(USER_PLUGIN_DIR)
 
         self.acoustidmanager = AcoustIDManager()
         self.browser_integration = BrowserIntegration()
@@ -208,12 +252,16 @@ class Tagger(QtGui.QApplication):
         self.albums = {}
         self.release_groups = {}
         self.mbid_redirects = {}
-        self.unmatched_files = UnmatchedFiles()
+        self.unclustered_files = UnclusteredFiles()
         self.nats = None
         self.window = MainWindow()
         self.exit_cleanup = []
         self.stopping = False
 
+        # Load release version information
+        if self.autoupdate_enabled:
+            self.updatecheckmanager = UpdateCheckManager(parent=self.window)
+
     def register_cleanup(self, func):
         self.exit_cleanup.append(func)
 
@@ -221,16 +269,41 @@ class Tagger(QtGui.QApplication):
         for f in self.exit_cleanup:
             f()
 
-    def debug(self, debug):
-        if self._debug == debug:
-            return
-        if debug:
-            log.log_levels = log.log_levels | log.LOG_DEBUG
-            log.debug("Debug mode on")
+    def set_log_level(self, level):
+        self._debug = level == logging.DEBUG
+        log.set_level(level)
+
+    def mb_login(self, callback, parent=None):
+        scopes = "profile tag rating collection submit_isrc submit_barcode"
+        authorization_url = self.webservice.oauth_manager.get_authorization_url(scopes)
+        webbrowser2.open(authorization_url)
+        if not parent:
+            parent = self.window
+        authorization_code, ok = QtWidgets.QInputDialog.getText(parent,
+            _("MusicBrainz Account"), _("Authorization code:"))
+        if ok:
+            self.webservice.oauth_manager.exchange_authorization_code(
+                authorization_code, scopes,
+                partial(self.on_mb_authorization_finished, callback))
         else:
-            log.debug("Debug mode off")
-            log.log_levels = log.log_levels & ~log.LOG_DEBUG
-        self._debug = debug
+            callback(False)
+
+    def on_mb_authorization_finished(self, callback, successful):
+        if successful:
+            self.webservice.oauth_manager.fetch_username(
+                partial(self.on_mb_login_finished, callback))
+        else:
+            callback(False)
+
+    @classmethod
+    def on_mb_login_finished(self, callback, successful):
+        if successful:
+            load_user_collections()
+        callback(successful)
+
+    def mb_logout(self):
+        self.webservice.oauth_manager.revoke_tokens()
+        load_user_collections()
 
     def move_files_to_album(self, files, albumid=None, album=None):
         """Move `files` to tracks on album `albumid`."""
@@ -257,6 +330,7 @@ class Tagger(QtGui.QApplication):
             self.nats = NatAlbum()
             self.albums["NATS"] = self.nats
             self.album_added.emit(self.nats)
+            self.nats.item.setExpanded(True)
         return self.nats
 
     def move_file_to_nat(self, file, recordingid, node=None):
@@ -268,13 +342,16 @@ class Tagger(QtGui.QApplication):
             self.nats.update()
 
     def exit(self):
-        log.debug("Picard stopping")
+        if self.stopping:
+            return
         self.stopping = True
+        log.debug("Picard stopping")
         self._acoustid.done()
         self.thread_pool.waitForDone()
         self.save_thread_pool.waitForDone()
+        self.priority_thread_pool.waitForDone()
         self.browser_integration.stop()
-        self.xmlws.stop()
+        self.webservice.stop()
         self.run_cleanup()
         QtCore.QCoreApplication.processEvents()
 
@@ -303,13 +380,12 @@ class Tagger(QtGui.QApplication):
         if isinstance(event, thread.ProxyToMainEvent):
             event.run()
         elif event.type() == QtCore.QEvent.FileOpen:
-            f = str(event.file())
-            self.add_files([f])
+            self.add_files([event.file()])
             # We should just return True here, except that seems to
             # cause the event's sender to get a -9874 error, so
             # apparently there's some magic inside QFileOpenEvent...
             return 1
-        return QtGui.QApplication.event(self, event)
+        return super().event(event)
 
     def _file_loaded(self, file, target=None):
         if file is not None and not file.has_error():
@@ -360,6 +436,10 @@ class Tagger(QtGui.QApplication):
             if ignore_hidden and is_hidden(filename):
                 log.debug("File ignored (hidden): %r" % (filename))
                 continue
+            # Ignore .smbdelete* files which Applie iOS SMB creates by renaming a file when it cannot delete it
+            if os.path.basename(filename).startswith(".smbdelete"):
+                log.debug("File ignored (.smbdelete): %r", filename)
+                continue
             if ignoreregex is not None and ignoreregex.search(filename):
                 log.info("File ignored (matching %r): %r" % (pattern, filename))
                 continue
@@ -371,8 +451,8 @@ class Tagger(QtGui.QApplication):
         if new_files:
             log.debug("Adding files %r", new_files)
             new_files.sort(key=lambda x: x.filename)
-            if target is None or target is self.unmatched_files:
-                self.unmatched_files.add_files(new_files)
+            if target is None or target is self.unclustered_files:
+                self.unclustered_files.add_files(new_files)
                 target = None
             for file in new_files:
                 file.load(partial(self._file_loaded, target=target))
@@ -385,7 +465,7 @@ class Tagger(QtGui.QApplication):
 
     def _add_directory_recursive(self, path):
         ignore_hidden = config.setting["ignore_hidden_files"]
-        walk = os.walk(unicode(path))
+        walk = os.walk(path)
 
         def get_files():
             try:
@@ -404,7 +484,7 @@ class Tagger(QtGui.QApplication):
                     log.debug("Adding %(count)d files from '%(directory)r'" %
                               mparms)
                     self.window.set_statusbar_message(
-                        ungettext(
+                        ngettext(
                             "Adding %(count)d file from '%(directory)s' ...",
                             "Adding %(count)d files from '%(directory)s' ...",
                             number_of_files),
@@ -437,7 +517,7 @@ class Tagger(QtGui.QApplication):
             log.debug("Adding %(count)d files from '%(directory)r'" %
                       mparms)
             self.window.set_statusbar_message(
-                ungettext(
+                ngettext(
                     "Adding %(count)d file from '%(directory)s' ...",
                     "Adding %(count)d files from '%(directory)s' ...",
                     number_of_files),
@@ -464,29 +544,39 @@ class Tagger(QtGui.QApplication):
         if mimeData.hasUrls():
             BaseTreeView.drop_urls(mimeData.urls(), target)
 
-    def search(self, text, type, adv=False):
+    def search(self, text, search_type, adv=False, mbid_matched_callback=None, force_browser=False):
         """Search on the MusicBrainz website."""
+        search_types = {
+            'track': {
+                'entity': 'recording',
+                'dialog': TrackSearchDialog
+            },
+            'album': {
+                'entity': 'release',
+                'dialog': AlbumSearchDialog
+            },
+            'artist': {
+                'entity': 'artist',
+                'dialog': ArtistSearchDialog
+            },
+        }
+        if search_type not in search_types:
+            return
+        search = search_types[search_type]
         lookup = self.get_file_lookup()
-        if config.setting["builtin_search"]:
-            if type == "track" and not lookup.mbidLookup(text, 'recording'):
-                dialog = TrackSearchDialog(self.window)
-                dialog.search(text)
-                dialog.exec_()
-            elif type == "album" and not lookup.mbidLookup(text, 'release'):
-                dialog = AlbumSearchDialog(self.window)
-                dialog.search(text)
-                dialog.exec_()
-            elif type == "artist" and not lookup.mbidLookup(text, 'artist'):
-                dialog = ArtistSearchDialog(self.window)
+        if config.setting["builtin_search"] and not force_browser:
+            if not lookup.mbid_lookup(text, search['entity'],
+                                      mbid_matched_callback=mbid_matched_callback):
+                dialog = search['dialog'](self.window)
                 dialog.search(text)
                 dialog.exec_()
         else:
-            getattr(lookup, type + "Search")(text, adv)
+            lookup.search_entity(search['entity'], text, adv, mbid_matched_callback=mbid_matched_callback)
 
     def collection_lookup(self):
         """Lookup the users collections on the MusicBrainz website."""
         lookup = self.get_file_lookup()
-        lookup.collectionLookup(config.persist["oauth_username"])
+        lookup.collection_lookup(config.persist["oauth_username"])
 
     def browser_lookup(self, item):
         """Lookup the object's metadata on the MusicBrainz website."""
@@ -496,11 +586,11 @@ class Tagger(QtGui.QApplication):
         if isinstance(item, DataObject):
             itemid = item.id
             if isinstance(item, Track):
-                lookup.recordingLookup(itemid)
+                lookup.recording_lookup(itemid)
             elif isinstance(item, Album):
-                lookup.albumLookup(itemid)
+                lookup.album_lookup(itemid)
         else:
-            lookup.tagLookup(
+            lookup.tag_lookup(
                 metadata["albumartist"] if item.is_album_like() else metadata["artist"],
                 metadata["album"],
                 metadata["title"],
@@ -518,25 +608,26 @@ class Tagger(QtGui.QApplication):
         for file in files:
             file.save()
 
-    def load_album(self, id, discid=None):
-        id = self.mbid_redirects.get(id, id)
-        album = self.albums.get(id)
+    def load_album(self, album_id, discid=None):
+        album_id = self.mbid_redirects.get(album_id, album_id)
+        album = self.albums.get(album_id)
         if album:
-            log.debug("Album %s already loaded.", id)
+            log.debug("Album %s already loaded.", album_id)
+            album.add_discid(discid)
             return album
-        album = Album(id, discid=discid)
-        self.albums[id] = album
+        album = Album(album_id, discid=discid)
+        self.albums[album_id] = album
         self.album_added.emit(album)
         album.load()
         return album
 
-    def load_nat(self, id, node=None):
+    def load_nat(self, nat_id, node=None):
         self.create_nats()
-        nat = self.get_nat_by_id(id)
+        nat = self.get_nat_by_id(nat_id)
         if nat:
-            log.debug("NAT %s already loaded.", id)
+            log.debug("NAT %s already loaded.", nat_id)
             return nat
-        nat = NonAlbumTrack(id)
+        nat = NonAlbumTrack(nat_id)
         self.nats.tracks.append(nat)
         self.nats.update(True)
         if node:
@@ -545,14 +636,14 @@ class Tagger(QtGui.QApplication):
             nat.load()
         return nat
 
-    def get_nat_by_id(self, id):
+    def get_nat_by_id(self, nat_id):
         if self.nats is not None:
             for nat in self.nats.tracks:
-                if nat.id == id:
+                if nat.id == nat_id:
                     return nat
 
-    def get_release_group_by_id(self, id):
-        return self.release_groups.setdefault(id, ReleaseGroup(id))
+    def get_release_group_by_id(self, rg_id):
+        return self.release_groups.setdefault(rg_id, ReleaseGroup(rg_id))
 
     def remove_files(self, files, from_parent=True):
         """Remove files from the tagger."""
@@ -575,6 +666,16 @@ class Tagger(QtGui.QApplication):
             self.nats = None
         self.album_removed.emit(album)
 
+    def remove_nat(self, track):
+        """Remove the specified non-album track."""
+        log.debug("Removing %r", track)
+        self.remove_files(self.get_files_from_objects([track]))
+        self.nats.tracks.remove(track)
+        if not self.nats.tracks:
+            self.remove_album(self.nats)
+        else:
+            self.nats.update(True)
+
     def remove_cluster(self, cluster):
         """Remove the specified cluster."""
         if not cluster.special:
@@ -592,6 +693,8 @@ class Tagger(QtGui.QApplication):
         for obj in objects:
             if isinstance(obj, File):
                 files.append(obj)
+            elif isinstance(obj, NonAlbumTrack):
+                self.remove_nat(obj)
             elif isinstance(obj, Track):
                 files.extend(obj.linked_files)
             elif isinstance(obj, Album):
@@ -604,7 +707,7 @@ class Tagger(QtGui.QApplication):
                     }
                 )
                 self.remove_album(obj)
-            elif isinstance(obj, UnmatchedFiles):
+            elif isinstance(obj, UnclusteredFiles):
                 files.extend(list(obj.files))
             elif isinstance(obj, Cluster):
                 self.remove_cluster(obj)
@@ -614,15 +717,15 @@ class Tagger(QtGui.QApplication):
     def _lookup_disc(self, disc, result=None, error=None):
         self.restore_cursor()
         if error is not None:
-            QtGui.QMessageBox.critical(self.window, _(u"CD Lookup Error"),
-                                       _(u"Error while reading CD:\n\n%s") % error)
+            QtWidgets.QMessageBox.critical(self.window, _("CD Lookup Error"),
+                                           _("Error while reading CD:\n\n%s") % error)
         else:
             disc.lookup()
 
     def lookup_cd(self, action):
         """Reads CD from the selected drive and tries to lookup the DiscID on MusicBrainz."""
-        if isinstance(action, QtGui.QAction):
-            device = unicode(action.text())
+        if isinstance(action, QtWidgets.QAction):
+            device = action.text()
         elif config.setting["cd_lookup_device"] != '':
             device = config.setting["cd_lookup_device"].split(",", 1)[0]
         else:
@@ -633,7 +736,8 @@ class Tagger(QtGui.QApplication):
         self.set_wait_cursor()
         thread.run_task(
             partial(disc.read, encode_filename(device)),
-            partial(self._lookup_disc, disc))
+            partial(self._lookup_disc, disc),
+            traceback=self._debug)
 
     @property
     def use_acoustid(self):
@@ -663,18 +767,14 @@ class Tagger(QtGui.QApplication):
     def cluster(self, objs):
         """Group files with similar metadata to 'clusters'."""
         log.debug("Clustering %r", objs)
-        if len(objs) <= 1 or self.unmatched_files in objs:
-            files = list(self.unmatched_files.files)
+        if len(objs) <= 1 or self.unclustered_files in objs:
+            files = list(self.unclustered_files.files)
         else:
             files = self.get_files_from_objects(objs)
-        fcmp = lambda a, b: (
-            cmp(a.discnumber, b.discnumber) or
-            cmp(a.tracknumber, b.tracknumber) or
-            cmp(a.base_filename, b.base_filename))
         for name, artist, files in Cluster.cluster(files, 1.0):
             QtCore.QCoreApplication.processEvents()
             cluster = self.load_cluster(name, artist)
-            for file in sorted(files, fcmp):
+            for file in sorted(files, key=attrgetter('discnumber', 'tracknumber', 'base_filename')):
                 file.move(cluster)
 
     def load_cluster(self, name, artist):
@@ -693,12 +793,12 @@ class Tagger(QtGui.QApplication):
 
     def set_wait_cursor(self):
         """Sets the waiting cursor."""
-        QtGui.QApplication.setOverrideCursor(
+        super().setOverrideCursor(
             QtGui.QCursor(QtCore.Qt.WaitCursor))
 
     def restore_cursor(self):
         """Restores the cursor set by ``set_wait_cursor``."""
-        QtGui.QApplication.restoreOverrideCursor()
+        super().restoreOverrideCursor()
 
     def refresh(self, objs):
         for obj in objs:
@@ -718,7 +818,7 @@ class Tagger(QtGui.QApplication):
         log.debug("signal %i received", signum)
         # Send a notification about a received signal from the signal handler
         # to Qt.
-        self.signalfd[0].sendall("a")
+        self.signalfd[0].sendall(b"a")
 
     def sighandler(self):
         self.signalnotifier.setEnabled(False)
@@ -744,6 +844,10 @@ def process_picard_args():
                         help="location of the configuration file")
     parser.add_argument("-d", "--debug", action='store_true',
                         help="enable debug-level logging")
+    parser.add_argument("-N", "--no-restore", action='store_true',
+                        help="do not restore positions and/or sizes")
+    parser.add_argument("-P", "--no-plugins", action='store_true',
+                        help="do not load any plugins")
     parser.add_argument('-v', '--version', action='store_true',
                         help="display version information and exit")
     parser.add_argument("-V", "--long-version", action='store_true',
@@ -755,8 +859,8 @@ def process_picard_args():
 
 def main(localedir=None, autoupdate=True):
     # Some libs (ie. Phonon) require those to be set
-    QtGui.QApplication.setApplicationName(PICARD_APP_NAME)
-    QtGui.QApplication.setOrganizationName(PICARD_ORG_NAME)
+    QtWidgets.QApplication.setApplicationName(PICARD_APP_NAME)
+    QtWidgets.QApplication.setOrganizationName(PICARD_ORG_NAME)
 
     signal.signal(signal.SIGINT, signal.SIG_DFL)
 
@@ -767,5 +871,16 @@ def main(localedir=None, autoupdate=True):
         return longversion()
 
     tagger = Tagger(picard_args, unparsed_args, localedir, autoupdate)
+
+    # Initialize Qt default translations
+    translator = QtCore.QTranslator()
+    locale = QtCore.QLocale()
+    translation_path = QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath)
+    log.debug("Looking for Qt locale %s in %s", locale.name(), translation_path)
+    if translator.load(locale, "qtbase_", directory=translation_path):
+        tagger.installTranslator(translator)
+    else:
+        log.debug('Qt locale %s not available', locale.name())
+
     tagger.startTimer(1000)
     sys.exit(tagger.run())
